#include "Config.h"

#include <cstdio>
#include <cstring>
#include <string>
#include <vector>
#include "Exception.h"


namespace DFF {


Exception::Exception( const std::string& description ) {
  std::strncpy( m_name, "Exception", EC_NAME_LENGTH );
  std::strncpy( m_description, description.c_str(), EC_DESCRIPTION_LENGTH );
}

  
Exception::Exception( const std::string& name, const std::string& description ) {
  std::strncpy( m_name, name.c_str(), EC_NAME_LENGTH );
  std::strncpy( m_description, description.c_str(), EC_DESCRIPTION_LENGTH );
}


Exception::Exception( const Exception& exception ) {
  std::strncpy( m_name, exception.m_name, EC_NAME_LENGTH );
  std::strncpy( m_description, exception.m_description, EC_DESCRIPTION_LENGTH );
}


Exception& Exception::operator=( const Exception& exception ) {
  if ( this != &exception ) {
    std::strncpy( m_name, exception.m_name, EC_NAME_LENGTH );
    std::strncpy( m_description, exception.m_description, EC_DESCRIPTION_LENGTH );
  }
  
  return *this;
}


DetailedException::DetailedException( const std::string& description,
  const std::vector< std::string >& details )
  : Exception( "DetailedException", description ),
    m_details( details ) {
}


DetailedException::DetailedException( const std::string& description,
  const std::string& details )
  : Exception( "DetailedException", description ) {
  m_details.push_back( details );
}


DetailedException::DetailedException( const std::string& name, const std::string& description,
  const std::vector< std::string >& details )
  : Exception( name, description ),
    m_details( details ) {
}


DetailedException::DetailedException( const DetailedException& exception )
  : Exception( exception ),
    m_details( exception.m_details ) {
}


DetailedException& DetailedException::operator=( const DetailedException& exception ) {
  if ( this != &exception ) {
    Exception::operator=(exception);

    m_details = exception.m_details;
  }

  return *this;
}


}
