#include "Config.h"

#include <cstring>
#include <streambuf>
#include <string>

#if defined(_DFF_ON_WINDOWS_)
# include <direct.h>
# include <shellapi.h>
# include "Shlwapi.h"
#else
# include <unistd.h>
#endif

#include "Exception.h"
#include "ExceptionUtils.h"
#include "FilesystemUtils.h"
#include "SystemUtils.h"


#define TEMP_DIR_ENV_VARIABLE_NAME "DINAMICA_TEMP_DIR"


namespace DFF {


namespace {


std::string getUserTemporaryDirectory() {
  return DFF::SystemUtils::getEnvironmentVariableValue( 
      TEMP_DIR_ENV_VARIABLE_NAME );
}

std::string getOSTemporaryDirectory() {
  char tmpBuffer[ 1024 ] = { 0 };
  ::GetTempPath( 1024, tmpBuffer );

  return std::string( tmpBuffer );
}

std::string getSystemTemporaryDirectory() {
  const auto userDirectory( getUserTemporaryDirectory() );
  const auto tmpDirectory( getOSTemporaryDirectory() );
  return userDirectory.empty() ? tmpDirectory : userDirectory;
}


} // namespace ''


#if defined(_DFF_ON_WINDOWS_)
# define getCurrentDirectory _getcwd
#else
# define MAX_PATH 255
# define getCurrentDirectory getcwd
#endif

std::string FilesystemUtils::getCompleteTemporaryPath( const std::string& baseName ) {
	return ( getSystemTemporaryDirectory() + "\\" + baseName );
}

std::string FilesystemUtils::getTemporaryFilename() {
  char tempFilenameBuffer[ 1024 ] = { 0 };
  if ( ::GetTempFileName( getSystemTemporaryDirectory().c_str(),
        "EGO", 0, tempFilenameBuffer ) == 0 ) {

		std::string errMsg;
		SystemUtils::getLastSystemError( errMsg );

    throwException( DFF::Exception, "Failed to get temporary file: " <<
      errMsg );
  }

  return std::string( tempFilenameBuffer );
}


}

