#include "Config.h"

#define _DFF_CPP_LIBRARY_ BaseLib

#include <string>
#if defined(_DFF_ON_WINDOWS_)
# include <windows.h>
# include <Shlobj.h>
#elif defined(_DFF_ON_LINUX_)
# include <cstring>
# include <boost/config/warning_disable.hpp>
# include <boost/spirit/include/qi.hpp>
# include <linux/unistd.h>
#endif

#include <cstdlib>
#include <sstream>

#include "Exception.h"
#include "ExceptionUtils.h"
#include "Log.h"
#include "SystemUtils.h"


namespace {


template < typename T >
std::string toString( T param ) {
  std::stringstream __str;
  __str << param;

  return __str.str();
}


}


namespace DFF {


bool SystemUtils::getLastSystemError( std::string& message ) {
#if defined(_DFF_ON_WINDOWS_)

  DWORD errorCode = GetLastError();
  LPVOID messageBuffer = 0;

  if ( errorCode != 0 ) {
    logDebug2( "System error detected. Retrieving last error message:" );

    if ( ::FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM |
      FORMAT_MESSAGE_IGNORE_INSERTS, NULL, errorCode,
      MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) & messageBuffer,
      0, NULL ) != 0 ) {
      message = std::string( (LPCTSTR) messageBuffer );

      LocalFree( messageBuffer );

    } else
      message = "Error #" + toString( errorCode );

    logDebug2( "Last error message retrieved successfully: " << message );

    return true;

  } else
    return false;

#else

  int errorCode = errno;
  if ( errorCode != 0 ) {
    logDebug2( "System error detected. Retrieving last error message:" );

    std::stringstream errorMsg;
    errorMsg << "Error #" << errorCode << ", " << ::strerror( errorCode );
    logDebug2( "Last error message retrieved successfully: " << errorMsg.str() );

    return true;

  } else
    return false;

#endif
}

std::string SystemUtils::getEnvironmentVariableValue( const std::string& variableName ) {

  const char* variableValue = std::getenv( variableName.c_str() );
  if ( variableValue != nullptr )
    return std::string( variableValue );
  else
    return std::string();
}


} /* DFF */

